<?php
/* Copyright 2007, 2008 Xavier 'Xr' Dalem
 * 
 * This file is part of Demyo. Demyo is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 * 
 * Demyo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU GPLv3 for more details.
 */
/** Provides common but non standard IO operations on files */
class FileComponent extends Object {
	private function error($message, $level = LOG_ERROR) {
		$message = __($message, true);
		$this->log($message, $level);
		if ($level == LOG_ERROR)
			$this->cakeError('error', array(array('message' => $message, 'base' => '', 'code' => '', 'name' => __('File error', true))));
		return false;
	}
	
	/** Recursively changes the mode of a file or directory.
	 *
	 * @param string $filename The path to the file or directory
	 * @param int $mode The mode to set
	 * @param int $dirMode The mode to set for directories. If not specified,
	 * it is based on the normal mode, adding execution if the file can be read.
	 * For example, a mode of -rw-r---- will give a dirMode of -rwxr-x---.
	 * Recursion is done depth-first. If a directory is not accessible, it won't
	 * be recursed. On the other hand, if a directory is accessible and when
	 * trying to remove access to it, sub-files will be chmod'ed.
	 */
	public function chmod($filename, $mode = 0777, $dirMode = -1) {
		if ($dirMode < 0) {
			$dirMode = $mode;
			
			if ($mode & 0400) $dirMode |= 0100;
			if ($mode & 0040) $dirMode |= 0010;
			if ($mode & 0004) $dirMode |= 0001;
		}
		
		if (is_file($filename))
			return chmod($filename, $mode);
		
		else if (is_dir($filename)) {
			foreach (new DirectoryIterator($filename) as $entry) {
				if ($entry->isDot())
					continue;
				if (!$this->chmod($entry->getPathname(), $mode, $dirMode))
					return false;
			}
			
			return chmod($filename, $dirMode);
		}
		else
			// Skip special files
			return true;
		
		assert(false); // Should be unreachable
	}
	
	/** Recursively deletes a file or directory.
	 *
	 * @param string $filename The path to the file or directory
	 * @param boolean leaveRoot If true, the passed file will not be deleted
	 * if it's a directory (only its contents).
	 * @return boolean true on success, false on failure
	 */
	public function rm($filename, $leaveRoot = false) {
		if (is_file($filename))
			return unlink($filename);
		else if (is_dir($filename)) {
			foreach (new DirectoryIterator($filename) as $entry) {
				if ($entry->isDot())
					continue;
				if (!$this->rm($entry->getPathname()))
					return false;
			}
			if (! $leaveRoot)
				return rmdir($filename);
		} else
			return $this->error('Entry to delete is neither a file nor a directory');
		
		return true;
	}
	
	/** Moves the content of a directory.
	 * Compared with PHP's rename(), this function is slower but keeps
	 * permissions and ownership on the destination directory, and doesn't
	 * require write permissions on the parent of the destination directory
	 * (since no directory is created or deleted).
	 *
	 * @param string $source Path to source directory
	 * @param string $destination Path to destination directory. If it
	 * doesn't exist, it will be created (thus requiring write access on parent
	 * directory, contrary to what was stated above)
	 */
	public function moveDirContent($source, $destination) {
		if (! is_dir($source))
			return $this->error('Source directory does not exist');
		if (file_exists($destination) && ! is_dir($destination))
			return $this->error('Destination directory is actually not a directory');
		else if (!file_exists($destination) && ! mkdir($destination))
			return $this->error('Failed to create destination directory');
		
		foreach (new DirectoryIterator($source) as $entry) {
			if ($entry->isDot())
				continue;
			
			// No need to recurse here: rename does exactly what we want it to.
			// Remember: we just want to keep permission on $source, the contents
			// can be moved normally
			if (!rename($entry->getPathname(), $destination.DS.$entry->getFilename()))
				return false;
		}
		return true;
	}
	
	/** Recursively adds a directory to a zip archive.
	 *
	 * @param ZipArchive $zip The archive to write in (opened)
	 * @param string $dir The path to the directory to add
	 * @param string $base The path where to put the directory in the zip
	 * (defaults to the name of the directory to add)
	 */
	public function addDirToZip(&$zip, $dir, $base = null) {
		if (! is_dir($dir))
			return $this->error('Cannot add non-dir in addDirToZip');
		
		if ($base == null)
			$base = basename($dir);
		$zip->addEmptyDir($base);

        foreach (new DirectoryIterator($dir) as $entry) {
        	if ($entry->isDot())
        		continue;
        	
            if ($entry->isFile())
            	$zip->addFile($entry->getPathname(), $base.'/'.$entry->getFilename());
            else if ($entry->isDir())
            	$this->addDirToZip($zip, $entry->getPathname(), $base.'/'.$entry->getFilename());
            else
            	$this->error('Skipped an entry that is neither a file nor a directory during zipping', LOG_WARNING);
        }
	}
	
	/** Compares 2 paths.
	 * Taken from AMP
	 * @param path1 The first path, relative to the caller
	 * @param path2 The second path, relative to the caller
	 * @return A positive number if path2 is below path1 in the directory
	 * hierarchy, 0 if paths are equal and a negative number if path2 is not
	 * below path1, @c false on failure.
	 */
	public function dir_cmp($path1, $path2) {
		$r_path1 = realpath($path1);
		$r_path2 = realpath($path2);
		if ($r_path1 === false || $r_path2 === false)
			return false;

		if ($r_path1 == $r_path2)
			return 0;

		$r_path1 = explode(DS, $r_path1);
		$r_path2 = explode(DS, $r_path2);

		$i = 0;
		for (; $i < count($r_path1) && $i < count($r_path2); $i++) {
			if ($r_path1[$i] != $r_path2[$i]) // "fork" in hierarchy
				return -1;
		}

		// We reached end of one path without "fork". The longer path is thus
		// the one contained in the other
		if ($i == count($r_path1))
			return 1;
		else
			return -1;
	}
	
	/** Returns a sorted array of all files in given path matching given expression.
	 * . and .. directories are omitted.
	 * Taken from AMP
	 * @return A bi-dimensional array (both sub-arrays might be empty), or null
	 * in case of error
	 */
	function getFileList($path, $regex = '/.*/') {
		if (substr($path, -1) != '/')
			$path .= '/';

		$list['files'] = array();
		$list['dirs'] = array();

		$dh = @opendir($path);
		if ($dh === false)
			return null;

		while (($entry = readdir($dh)) !== false) {
			// Depending on implementation, $file may or may not contain full path
			$entry = $path.basename($entry);

			if (is_dir($entry) && basename($entry) != '.' && basename($entry) != '..')
				$list['dirs'][] = $entry;

			else if (is_file($entry)
					&& preg_match($regex, basename($entry)))
				$list['files'][] = $entry;
		}

		closedir($dh);

		if (! (natcasesort($list['files']) && natcasesort($list['dirs'])))
			return null;

		return $list;
	}
}
?>