<?php
/* Copyright 2007, 2008 Xavier 'Xr' Dalem
 * 
 * This file is part of Demyo. Demyo is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 * 
 * Demyo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU GPLv3 for more details.
 */
class AlbumsBorrower extends AppModel {
	public $name = 'AlbumsBorrower';
	public $actsAs = array('Containable', 'SimpleXML' => array('container' => 'loan-history', 'element' => 'loan'));
	public $belongsTo = array('Album', 'Borrower');
	private $stillLendedCondition =
		array('OR' => array('`return_date` IS NULL', array('return_date' => '0000-00-00')));

	/* Cake doesn't support compound primary keys (see cake bug #4108), and I'm not
	 * making concessions since they are perfectly legit.
	 * Thus, we can't use the all simple usual habtm automatic handling and
	 * will have to code a few functions by hand */
		
	/** Lends a set of albums to a given borrower.
	 *
	 * @param integer $borrower The borrower ID
	 * @param array $albums An array of IDs of albums to lend
	 * @param string $date An SQL string to use for the borrow date
	 * @throws Exception if the loan failed. The exception message is localised
	 */
	public function lend($borrower, $albums, $date = null) {
		if ($date == null)
			$date = date('Y-m-d');
		
		// Check that none of the albums are lended
		//if($this->findCount('album_id IN ('.implode(',',$albums).') AND (return_date IS NULL OR return_date=\'0000-00-00\')', -1))
		if($this->find('count', array('conditions' => array(
					array('album_id' => $albums),
					$this->stillLendedCondition
				), 'recursive' => -1
			)))
			throw new Exception(__('One or more of the specified albums are already lended', true));

		// OK, insert the new ones
		$db =& ConnectionManager::getDataSource($this->useDbConfig);
		foreach($albums as $album)
			$db->create($this, array('borrower_id', 'album_id', 'borrow_date'),
				array($borrower, $album, $date));
	}
	
	/** Indicate that a set of albums has been returned.
	 *
	 * @param array $albums An array of IDs of albums that are back in the library
	 * @param string $date An SQL string to use for the return date
	 * @throws Exception if the return failed. The exception message is localised.
	 */
	public function getBack($albums, $date = null) {
		if ($date == null)
			$date = date('Y-m-d');
		
		// Since this operation won't do anything to albums that aren't lended,
		// there's no need to check if all of them are lended
		$db =& ConnectionManager::getDataSource($this->useDbConfig);
		$db->update($this, array('return_date'), array("'$date'"),
			'album_id IN ('.implode(',',$albums).') AND (return_date IS NULL OR return_date=\'0000-00-00\')');
	}
	
	/** Gets the number of albums currently lended. */
	public function findLendedCount() {
		return $this->find('count',
			array('conditions' => $this->stillLendedCondition, 'recursive' => -1));
	}
	
	/** Find out whether a borrower currently has albums.
	 *
	 * @param integer The borrower id
	 * @return boolean true if the borrower currently has at least one album.
	 */
	public function hasAlbums($id) {
		return $this->find('count',
			array('conditions' => array($this->stillLendedCondition, array('borrower_id' => $id))
			, 'recursive' => -1)) > 0;
	}
	
	public function getLoans($fromBorrower = false, $showAll = false, $page = 1, $limit = 30) {
		$conditions = array();
		
		if ($fromBorrower !== false)
			$conditions[] = array('AlbumsBorrower.borrower_id' => $fromBorrower);
		
		if (! $showAll)
			$conditions[] = $this->stillLendedCondition;
		
		$this->contain(array(
			'Album' => array('id', 'title', 'number', 'cycle'),
			'Album.Series' => array('id', 'name'),
			'Borrower'
		));
		return $this->find('all',
			array('conditions' => $conditions,
				'fields' => array('AlbumsBorrower.*'),
				'order' => 'AlbumsBorrower.borrow_date DESC',
				'limit' => $limit,
				'page' => $page));
	}
	
	public function getLoansCount($fromBorrower = null, $showAll = false) {
		$conditions = array();
		
		if ($fromBorrower !== null)
			$conditions[] = array('AlbumsBorrower.borrower_id' => $fromBorrower);
		
		if (! $showAll)
			$conditions[] = $this->stillLendedCondition;
		
		return $this->find('count', array('conditions' => $conditions, 'recursive' => -1));
	}
}


?>
