<?php
/* Copyright 2007, 2008 Xavier 'Xr' Dalem
 * 
 * This file is part of Demyo. Demyo is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 * 
 * Demyo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU GPLv3 for more details.
 */
class ChartHelper extends AppHelper {
	private static $CHART_MAX_SIZE=1000;
	private static $CHART_MAX_AREA=300000;

	public $helpers = array('Html', 'Javascript', 'jQuery');
	private $renderer;
	
	/** Sets up the chart renderer depending on user preferences */
	public function beforeRender() {
		$method = '';
		if (($method = Configure::read('Demyo.View_chartMethod')) === null)
			$method = 'Flash';
		
		switch ($method) {
			case 'Google':
				$this->renderer = new GoogleChartRenderer('bvg');
				break;
			case 'Flash':
				$this->renderer = new OpenFlashChartRenderer($this->Javascript);
				break;
			default:
				trigger_error('Invalid charting method: '.$method, E_USER_ERROR);
		}
		
		$this->renderer->beforeRender();
	}
	
	public function afterRender() {
		$this->renderer->afterRender();
	}
	
	/** Generates a chart with auto-selected renderer and returns HTML for it.
	 *
	 * @param data An array of data sets. Each set is itself an array of
	 * <em>numberic</em> values.
	 * @param labels An array of labels (indexed by 'x' and 'y'), preferably of
	 * the same length as the longest set
	 * @param params Additional optional params:
	 * x_grid and y_grid: to set the vertical and horizontal grid spacing in percent).
	 * label_orientation: horizontal (H), vertical (V), parallel to axis (//), perpendicular to axis (T)
	 * @param width Chart image width (must be lower than 1000)
	 * @param height Chart image height (must be lower than 1000, total area
	 * must not exceed 300000 pixels)
	 * @return The HTML to include the chart.
	 */
	public function chart($data, $labels = array(), $params = array(), $width = 500, $height = 300) {
		// Check size
		if ($width > self::$CHART_MAX_SIZE || $height > self::$CHART_MAX_SIZE
			|| $width*$height > self::$CHART_MAX_AREA) {
			trigger_error("Invalid chart size", E_USER_WARNING);
			return '';
		}
		
		if (! isset($params['y_grid']))
			$params['y_grid'] = 10;
		
		return $this->renderer->chart($data, $labels, $width, $height, $params);
	}
}

abstract class ChartRenderer {
	const BG_COLOR = 'ffffff';
	const AXIS_COLOR = '666666';
	const GRID_COLOR = 'dddddd';
	const BAR_COLOR = '73a5d6';
	/** Recommended bar width, incl. padding */
	const RECOM_BAR_WIDTH = 45;
	
	public function afterRender() {}
	public function beforeRender() {}
	public abstract function chart($data, $labels, $width, $height, $params);
}

class GoogleChartRenderer extends ChartRenderer {
	private static $CHART_URL = 'http://chart.apis.google.com/chart?';
	
	private $type = '';
	
	public function __construct($type) {
		$this->type = $type;
	}
	
	/** Generates a chart with Google's Chart API and returns an url
	 * for it.
	 *
	 * Maximum is detected automatically and y-scale will be set accordingly.
	 *
	 * Ampersands in the returned url are properly encoded.
	 *
	 * @return HTML for the Google chart image
	 */
	public function chart($data, $labels, $width, $height, $params) {
		$url = self::$CHART_URL;
		
		$url.= "chs={$width}x{$height}";
		
		// Normalize data
		$max = max2d($data);
		$ndata = $this->normalize($data, 100/$max);
		
		// Data sets
		$url .= '&amp;chd=t:';
		$i = 0; $ilen = sizeof($ndata);
		foreach ($ndata as $skey => $set) {
			$j = 0; $jlen = sizeof($set);
			foreach($set as $key => $value) {
				$url .= number_format($value, 1, '.', ''); // locale independent
				if ($j++ < $jlen-1)
					$url .= ',';
			}
			if ($i++ < $ilen-1)
				$url .= '|';
		}
		
		// Type
		$url .= '&amp;cht='.$this->type;
		
		// Labels
		$url .= '&amp;chxt=y';
		if (isset($labels['x'])) {
			$url .= ',x&amp;chxl=1:';
			foreach($labels['x'] as $item)
				$url .= "|$item";
		}
		$url .= '&amp;chxr=0,0,'.$max;
		
		// Grid
		if (isset($params['x_grid']) && isset($params['y_grid']))
			$url .= "&amp;chg={$params['x_grid']},{$params['y_grid']},1,5";
		
		return "<img src=\"$url\" alt=\"Google chart\" />";
	}
	
	private function normalize($data, $factor) {
		$ret = array();
		
		foreach ($data as $skey => $set)
			foreach ($set as $key => $value)
				$ret[$skey][$key] = $value*$factor;
		
		return $ret;
	}
}

class OpenFlashChartRenderer extends ChartRenderer {
	private $numCharts = 0;
	private $Javascript = null;
	
	public function __construct(&$javascriptHelper) {
		$this->Javascript = $javascriptHelper;
		$this->Javascript->link('swfobject', false);
		$this->Javascript->link('json2', false);
		
		if (!App::import('Vendor', 'OFC_Chart', array('file' => 'OpenFlashChart'.DS.'OFC_Chart.php')))
			trigger_error('Failed to load OFC library', E_USER_ERROR);
	}
	
	/** Generates a chart with Open Flash Chart.
	 *
	 * @return HTML for the Google chart image
	 */
	public function chart($data, $labels, $width, $height, $params) {
		$this->numCharts++;
		
		if (! isset($params['label_orientation']))
			$params['label_orientation'] = 'T';
		
		$maxLength = count2d($data);
		
		$chart = '';
		
		// We don't need a really large graph if we don't have a lot to show
		if ($width > $maxLength*self::RECOM_BAR_WIDTH)
			$width = $maxLength*self::RECOM_BAR_WIDTH;
			
			// Print initialization functions on the first time
		if ($this->numCharts == 1) {
			$chart .= $this->Javascript->codeBlock(
				"var chartData = {};
function ofc_get_data(id) {
	eval('var foo = chartData.'+id);
	return JSON.stringify(foo);
}"
			);
		}
		
		// Placeholder
		$chart .= "<div class=\"spinner\" id=\"ofc-delayed-{$this->numCharts}\">Loading chart...</div>\n";
		
		// Create chart canvas
		$ofc = new OFC_Chart();
		$ofc->set_title(new OFC_Elements_Title());
		$ofc->set_bg_colour('#'.self::BG_COLOR);
		
		// Y Axis
		$y = new OFC_Elements_Axis_Y();
		$y->set_colour('#'.self::AXIS_COLOR);
		$max = max2d($data);
		if ($params['y_grid'] == 0) { // Hide grid, but show labels
			$y->set_grid_colour('#'.self::BG_COLOR);
			$params['y_grid'] = 20;
		} else
			$y->set_grid_colour('#'.self::GRID_COLOR);
		$y->set_range(0, max2d($data), (int)($max*$params['y_grid']/100));
		$ofc->set_y_axis($y);
		
		// X Axis
		$x = new OFC_Elements_Axis_X();
		$x->set_colour('#'.self::AXIS_COLOR);
		if ($params['x_grid'] == 0)
			$x->set_grid_colour('#'.self::BG_COLOR);
		else if ($params['x_grid'] > 0) // -1 to show on every step
			$x->set_steps($maxLength*$params['y_grid']/100);
		
		if (isset($labels['x'])) {
			$xLabels = new OFC_Elements_Axis_X_Label_Set();
			$xLabels->set_labels($labels['x']);
			$xLabels->rotate = $this->getOrientation('x', $params['label_orientation']);
			$x->set_labels($xLabels);
		}
		$ofc->set_x_axis($x);
		
		// Actual data
		foreach ($data as $set) {
			$bar = new OFC_Charts_Bar();
			$bar->set_colour('#'.self::BAR_COLOR);
			$bar->set_values($set);
			$ofc->add_element($bar);
		}
		
		$json = $ofc->toPrettyString();
		$chartDataIndex = $this->numCharts-1;
		// echo "<pre>$json</pre>"; // DEBUG
		$chart .= $this->Javascript->codeBlock(
"swfobject.embedSWF('".Router::url('/flash/open-flash-chart.swf')."', 'ofc-delayed-{$this->numCharts}', '$width', '$height', '9.0.0', '', { 'get-data':'ofc_get_data', 'id':'ofc_chart_{$this->numCharts}' });
chartData.ofc_chart_{$this->numCharts} = $json;\n");
		
		return $chart;
	}
	public function _chart($data, $labels, $width, $height, $params) {
		if (! isset($params['label_orientation']))
			$params['label_orientation'] = 'T';
		
		$maxLength = count2d($data);
		
		// We don't need a really large graph if we don't have a lot to show
		if ($width > $maxLength*self::RECOM_BAR_WIDTH)
			$width = $maxLength*self::RECOM_BAR_WIDTH;
		
		$chart = '';
		
		// Print initialization functions on the first time
		if ($this->numCharts++ == 0) {
			$chart .= $this->Javascript->codeBlock(
"var chartData = new Array();
var initializedCharts = 0;

// Hack to make OFC load multiple data without showing an overlayed error message
// See http://sourceforge.net/forum/forum.php?thread_id=2088658&forum_id=716572
function ofc_ready() {
	if (++initializedCharts >= TOTAL_NUM_CHARTS) {
		window.setTimeout('loadData()', 1000)
	}
}
function loadData() {
	for (var i = 0; i < TOTAL_NUM_CHARTS; i++) {
			findSWF('ofc-delayed-'+(i+1)).load(JSON.stringify(chartData[i]));
	}
}
function findSWF(movieName) {
  if (navigator.appName.indexOf('Microsoft') != -1) {
    return window[movieName];
  } else {
    return document[movieName];
  }
}
"
			);
		}
		
		// Placeholder
		$chart .= "<div class=\"spinner\" id=\"ofc-delayed-{$this->numCharts}\">Loading chart...</div>\n";
		
		// Create chart canvas
		$ofc = new open_flash_chart();
		$ofc->set_title(new title());
		$ofc->set_bg_colour('#'.self::BG_COLOR);
		
		// Y Axis
		$y = new y_axis();
		$y->set_colour('#'.self::AXIS_COLOR);
		$max = max2d($data);
		if ($params['y_grid'] == 0) { // Hide grid, but show labels
			$y->set_grid_colour('#'.self::BG_COLOR);
			$params['y_grid'] = 20;
		} else
			$y->set_grid_colour('#'.self::GRID_COLOR);
		$y->set_range(0, max2d($data), (int)($max*$params['y_grid']/100));
		$ofc->set_y_axis($y);
		
		// X Axis
		$x = new x_axis();
		$x->set_colour('#'.self::AXIS_COLOR);
		if ($params['x_grid'] == 0)
			$x->set_grid_colour('#'.self::BG_COLOR);
		else if ($params['x_grid'] > 0) // -1 to show on every step
			$x->set_steps($maxLength*$params['y_grid']/100);
		
		if (isset($labels['x'])) {
			$xLabels = new x_axis_labels();
			$xLabels->set_labels($labels['x']);
			$xLabels->rotate = $this->getOrientation('x', $params['label_orientation']);
			$x->set_labels($xLabels);
		}
		$ofc->set_x_axis($x);
		
		// Actual data
		foreach ($data as $set) {
			$bar = new bar();
			$bar->set_colour('#'.self::BAR_COLOR);
			$bar->set_values($set);
			$ofc->add_element($bar);
		}
		
		$json = $ofc->toPrettyString();
		$chartDataIndex = $this->numCharts-1;
		// echo "<pre>$json</pre>"; // DEBUG
		$chart .= $this->Javascript->codeBlock(
"swfobject.embedSWF('".Router::url('/flash/open-flash-chart.swf')."', 'ofc-delayed-{$this->numCharts}', '$width', '$height', '9.0.0');
chartData[$chartDataIndex] = $json;\n");
		
		return $chart;
	}
	
	private function getOrientation($axis, $or) {
		if ($or == 'V'
			|| ($axis == 'x' && $or == 'T')
			|| ($axis == 'y' && $or == '//'))
			return 'vertical';
		else
			return 'horizontal';
	}
	
	public function afterRender() {
		if ($this->numCharts > 0)
			echo $this->Javascript->codeBlock(
				"var TOTAL_NUM_CHARTS = {$this->numCharts};"
			);
	}
}

?>